@@ -51,11 +51,11 @@ sub typeize {
 }
 
 
-foreach $file (<*.defs>) {
-	open(F,"<$file");
-	$_ .= join("",<F>);
-	close(F);
-}
+#foreach $file (<*.defs>) {
+#	open(F,"<$file");
+#	$_ .= join("",<F>);
+#	close(F);
+#}
 
 foreach $file (@ARGV) {
 	open(F,"<$file") || next;
@@ -221,7 +221,20 @@ foreach $node (parse_lisp($_)) {
 		$func{$node[1]} = $func;
 
 
+	} elsif ($node[0] eq "export-enum") {
+		warn "Cannot export unknown enum `$node[1]'\n" if not exists $enum{$node[1]};
+		$enum{$node[1]}->{export} = 1;
+	} elsif ($node[0] eq "export-boxed") {
+		warn "Cannot export unknown boxed `$node[1]'\n" if not exists $boxed{$node[1]};
+		$boxed{$node[1]}->{export} = 1;
+	} elsif ($node[0] eq "export-flags") {
+		warn "Cannot export unknown flags `$node[1]'\n" if not exists $flags{$node[1]};
+		$flags{$node[1]}->{export} = 1;
+	} elsif ($node[0] eq "export-struct") {
+		warn "Cannot export unknown struct `$node[1]'\n" if not exists $struct{$node[1]};
+		$struct{$node[1]}->{export} = 1;
 	}
+
 }
 
 delete $pointer{""};
@@ -238,7 +251,12 @@ delete $pointer{""};
 #print Dumper(\%func);
 #print Dumper(\%struct);
 
-do 'overrides.pl';
+foreach (@ARGV) {
+	if (-f "$_.opl") {
+		do "$_.opl";
+	}
+}
+#do 'overrides.pl';
 
 delete $object{""};
 delete $func{""};
@@ -338,7 +356,7 @@ EOT
 
 
 
-open(STDOUT,">typemap.gtk") or die "Unable to write to typemap.gtk: $!";
+open(STDOUT,">xtypemap") or die "Unable to write to xtypemap: $!";
 print "\n\n# Do not edit this file, as it is automatically generated by gendefs.pl\n\n";
 print "TYPEMAP\n";
 $i = 0;
@@ -446,6 +464,7 @@ EOT
 
 $i = 0;
 foreach (sort keys %enum) {
+	print "#define TYPE_$_\n";
 	print "#define pGE_$_ pGtkType[$i]\n";
 	print "#define pGEName_$_ pGtkTypeName[$i]\n";
 	print "#define newSV$_(v) newSVOptsHash(v, pGEName_$_, pGE_$_)\n";
@@ -458,6 +477,7 @@ foreach (sort keys %enum) {
 	$i++;
 }
 foreach (sort keys %flags) {
+	print "#define TYPE_$_\n";
 	print "#define pGF_$_ pGtkType[$i]\n";
 	print "#define pGFName_$_ pGtkTypeName[$i]\n";
 	# Generate arrays
@@ -471,6 +491,7 @@ foreach (sort keys %flags) {
 	$i++;
 }
 foreach (sort keys %boxed) {
+	print "#define TYPE_$_\n";
 	print "extern SV * newSV$_($_ * value);\n";
 	print "extern $_ * Sv$_(SV * value);\n";
 	print "typedef $_ * $boxed{$_}->{xsname};\n";
@@ -480,6 +501,7 @@ foreach (sort keys %boxed) {
 	}
 }
 foreach (sort keys %struct) {
+	print "#define TYPE_$_\n";
 	print "extern SV * newSV$_($_ * value);\n";
 	print "extern $_ * SvSet$_(SV * value, $_ * dest);\n";
 	print "#define Sv$_(value) SvSet$_((value), 0)\n";
@@ -492,6 +514,7 @@ foreach (sort keys %struct) {
 }
 foreach (sort keys %object) {
 	print "#ifdef $object{$_}->{cast}\n";
+	print "#define TYPE_$_\n";
 	print "typedef $_ * $object{$_}->{xsname};\n";
 	print "typedef $_ * $object{$_}->{xsname}_OrNULL;\n";
 	print "typedef $_ * $object{$_}->{xsname}_Sink;\n";
@@ -703,11 +726,11 @@ SV * GtkGetArg(GtkArg * a)
 EOT
 
 foreach (sort keys %enum) {
-	print "#ifdef $enum{$_}->{typename}\n";
+	print "#ifdef $enum{$_}->{typename}\n" unless $enum{$_}->{export};
 	print "			if (a->type == $enum{$_}->{typename})\n";
 	print "				result = newSV$_(GTK_VALUE_ENUM(*a));\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $enum{$_}->{export};
 }
 
 print <<"EOT";
@@ -717,11 +740,11 @@ print <<"EOT";
 EOT
 
 foreach (sort keys %flags) {
-	print "#ifdef $flags{$_}->{typename}\n";
+	print "#ifdef $flags{$_}->{typename}\n" unless $flags{$_}->{export};
 	print "			if (a->type == $flags{$_}->{typename})\n";
 	print "				result = newSV$_(GTK_VALUE_FLAGS(*a));\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $flags{$_}->{export};
 }
 
 print <<"EOT";
@@ -763,11 +786,11 @@ print <<"EOT";
 EOT
 		
 foreach (sort keys %struct) {
-	print "#ifdef $struct{$_}->{typename}\n";
+	print "#ifdef $struct{$_}->{typename}\n" unless $struct{$_}->{export};
 	print "			if (a->type == $struct{$_}->{typename})\n";
 	print "				result = newSV$_(GTK_VALUE_POINTER(*a));\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $struct{$_}->{export};
 }
 
 print <<"EOT";
@@ -785,11 +808,11 @@ print <<"EOT";
 EOT
 
 foreach (sort keys %boxed) {
-	print "#ifdef $boxed{$_}->{typename}\n";
+	print "#ifdef $boxed{$_}->{typename}\n" unless $boxed{$_}->{export};
 	print "			if (a->type == $boxed{$_}->{typename})\n";
 	print "				result = newSV$_(GTK_VALUE_BOXED(*a));\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $boxed{$_}->{export};
 }
 
 print <<"EOT";
@@ -875,11 +898,11 @@ void GtkSetArg(GtkArg * a, SV * v, SV * Class, GtkObject * Object)
 EOT
 
 foreach (sort keys %struct) {
-	print "#ifdef $struct{$_}->{typename}\n";
+	print "#ifdef $struct{$_}->{typename}\n" unless $struct{$_}->{export};
 	print "			if (a->type == $struct{$_}->{typename})\n";
 	print "				GTK_VALUE_POINTER(*a) = Sv$_(v);\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $struct{$_}->{export};
 }
 print <<"EOT";
 				goto d_fault;
@@ -889,11 +912,11 @@ print <<"EOT";
 EOT
 
 foreach (sort keys %enum) {
-	print "#ifdef $enum{$_}->{typename}\n";
+	print "#ifdef $enum{$_}->{typename}\n" unless $enum{$_}->{export};
 	print "			if (a->type == $enum{$_}->{typename})\n";
 	print "				GTK_VALUE_ENUM(*a) = Sv$_(v);\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $enum{$_}->{export};
 }
 print <<"EOT";
 				goto d_fault;
@@ -901,11 +924,11 @@ print <<"EOT";
 		case GTK_TYPE_FLAGS:
 EOT
 foreach (sort keys %flags) {
-	print "#ifdef $flags{$_}->{typename}\n";
+	print "#ifdef $flags{$_}->{typename}\n" unless $flags{$_}->{export};
 	print "			if (a->type == $flags{$_}->{typename})\n";
 	print "				GTK_VALUE_FLAGS(*a) = Sv$_(v);\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $flags{$_}->{export};
 }
 
 print <<"EOT";
@@ -917,11 +940,11 @@ print <<"EOT";
 			else
 EOT
 foreach (sort keys %boxed) {
-	print "#ifdef $boxed{$_}->{typename}\n";
+	print "#ifdef $boxed{$_}->{typename}\n" unless $boxed{$_}->{export};
 	print "			if (a->type == $boxed{$_}->{typename})\n";
 	print "				GTK_VALUE_BOXED(*a) = Sv$_(v);\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $boxed{$_}->{export};
 }
 
 print <<"EOT";
@@ -950,11 +973,11 @@ void GtkSetRetArg(GtkArg * a, SV * v, SV * Class, GtkObject * Object)
 EOT
 
 foreach (sort keys %enum) {
-	print "#ifdef $enum{$_}->{typename}\n";
+	print "#ifdef $enum{$_}->{typename}\n" unless $enum{$_}->{export};
 	print "			if (a->type == $enum{$_}->{typename})\n";
 	print "				*GTK_RETLOC_ENUM(*a) = Sv$_(v);\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $enum{$_}->{export};
 }
 print <<"EOT";
 				goto d_fault;
@@ -962,11 +985,11 @@ print <<"EOT";
 		case GTK_TYPE_FLAGS:
 EOT
 foreach (sort keys %flags) {
-	print "#ifdef $flags{$_}->{typename}\n";
+	print "#ifdef $flags{$_}->{typename}\n" unless $flags{$_}->{export};
 	print "			if (a->type == $flags{$_}->{typename})\n";
 	print "				*GTK_RETLOC_FLAGS(*a) = Sv$_(v);\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $flags{$_}->{export};
 }
 
 print <<"EOT";
@@ -976,11 +999,11 @@ print <<"EOT";
 EOT
 
 foreach (sort keys %struct) {
-	print "#ifdef $struct{$_}->{typename}\n";
+	print "#ifdef $struct{$_}->{typename}\n" unless $struct{$_}->{export};
 	print "			if (a->type == $struct{$_}->{typename})\n";
 	print "				GTK_VALUE_POINTER(*a) = Sv$_(v);\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $struct{$_}->{export};
 }
 
 print <<"EOT";
@@ -993,11 +1016,11 @@ print <<"EOT";
 			else
 EOT
 foreach (sort keys %boxed) {
-	print "#ifdef $boxed{$_}->{typename}\n";
+	print "#ifdef $boxed{$_}->{typename}\n" unless $boxed{$_}->{export};
 	print "			if (a->type == $boxed{$_}->{typename})\n";
 	print "				GTK_VALUE_BOXED(*a) = Sv$_(v);\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $boxed{$_}->{export};
 }
 
 print <<"EOT";
@@ -1028,11 +1051,11 @@ SV * GtkGetRetArg(GtkArg * a)
 EOT
 
 foreach (sort keys %enum) {
-	print "#ifdef $enum{$_}->{typename}\n";
+	print "#ifdef $enum{$_}->{typename}\n" unless $enum{$_}->{export};
 	print "			if (a->type == $enum{$_}->{typename})\n";
 	print "				result = newSV$_(*GTK_RETLOC_ENUM(*a));\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $enum{$_}->{export};
 }
 print <<"EOT";
 				goto d_fault;
@@ -1040,11 +1063,11 @@ print <<"EOT";
 		case GTK_TYPE_FLAGS:
 EOT
 foreach (sort keys %flags) {
-	print "#ifdef $flags{$_}->{typename}\n";
+	print "#ifdef $flags{$_}->{typename}\n" unless $flags{$_}->{export};
 	print "			if (a->type == $flags{$_}->{typename})\n";
 	print "				result = newSV$_(*GTK_RETLOC_FLAGS(*a));\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $flags{$_}->{export};
 }
 
 print <<"EOT";
@@ -1054,11 +1077,11 @@ print <<"EOT";
 EOT
 
 foreach (sort keys %struct) {
-	print "#ifdef $struct{$_}->{typename}\n";
+	print "#ifdef $struct{$_}->{typename}\n" unless $struct{$_}->{export};
 	print "			if (a->type == $struct{$_}->{typename})\n";
 	print "				result = newSV$_(GTK_VALUE_POINTER(*a));\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $struct{$_}->{export};
 }
 
 print <<"EOT";
@@ -1071,11 +1094,11 @@ print <<"EOT";
 			else
 EOT
 foreach (sort keys %boxed) {
-	print "#ifdef $boxed{$_}->{typename}\n";
+	print "#ifdef $boxed{$_}->{typename}\n" unless $boxed{$_}->{export};
 	print "			if (a->type == $boxed{$_}->{typename})\n";
 	print "				result = newSV$_(GTK_VALUE_BOXED(*a));\n";
 	print "			else\n";
-	print "#endif\n";
+	print "#endif\n" unless $boxed{$_}->{export};
 }
 
 print <<"EOT";
@@ -1266,7 +1289,7 @@ print "\n\n# Do not edit this file, as it is automatically generated by gendefs.
 
 print "\"\n";
 foreach (sort keys %object) {
-	print "$_.o\n";
+	print "$_.xs\n";
 }
 print "\"\n;\n";
 
