@@ -6,6 +6,8 @@
 use Gtk;
 use Gtk::Atoms;
 
+$gtk_1_0 = (Gtk->major_version < 1 or (Gtk->major_version == 1 and Gtk->minor_version < 1));
+
 sub bbox_widget_destroy {
 	my($widget, $todestroy) = @_;
 	
@@ -505,21 +507,21 @@ sub create_shapes {
 	$root_win = Gtk::Gdk::Window->new_foreign(Gtk::Gdk->ROOT_WINDOW());
 	
 	if (not defined $modeller) {
-		$modeller = shape_create_icon("Modeller.xpm", 440, 140, 0,0, -popup);
+		$modeller = shape_create_icon("xpm/Modeller.xpm", 440, 140, 0,0, -popup);
 		$modeller->signal_connect("destroy", \&Gtk::Widget::destroyed, \$modeller);
 	} else {
 		destroy $modeller;
 	}
 
 	if (not defined $sheets) {
-		$sheets = shape_create_icon("FilesQueue.xpm", 580,170, 0,0, -popup);
+		$sheets = shape_create_icon("xpm/FilesQueue.xpm", 580,170, 0,0, -popup);
 		$sheets->signal_connect("destroy", \&Gtk::Widget::destroyed, \$sheets);
 	} else {
 		destroy $sheets;
 	}
 
 	if (not defined $rings) {
-		$rings = shape_create_icon("3DRings.xpm", 460, 270, 25,25, -toplevel);
+		$rings = shape_create_icon("xpm/3DRings.xpm", 460, 270, 25,25, -toplevel);
 		$rings->signal_connect("destroy", \&Gtk::Widget::destroyed, \$rings);
 	} else {
 		destroy $rings;
@@ -538,7 +540,7 @@ sub create_wmhints {
 		
 		$wmhints_window->realize;
 		
-		($circles,$mask) = Gtk::Gdk::Pixmap->create_from_xpm($wmhints_window->window, $wmhints_window->style->white, "circles.xpm");
+		($circles,$mask) = Gtk::Gdk::Pixmap->create_from_xpm($wmhints_window->window, $wmhints_window->style->white, "xpm/circles.xpm");
 		
 		$wmhints_window->window->set_icon(undef, $circles, $mask);
 
@@ -870,22 +872,22 @@ sub make_toolbar {
 	
 	$toolbar = new Gtk::Toolbar('horizontal', 'both');
 	$button = $toolbar->append_item ( "Horizontal", "Horizontal toolbar layout",
-		"Toolbar/Horizontal", new_pixmap("test.xpm",$window, $color));
+		"Toolbar/Horizontal", new_pixmap("xpm/test.xpm",$window, $color));
 	$button->signal_connect('clicked', sub {$toolbar->set_orientation('horizontal')});
 	$button = $toolbar->append_item( "Vertical","Vertical toolbar layout",
-		"Toolbar/Vertical", new_pixmap("test.xpm",$window, $color));
+		"Toolbar/Vertical", new_pixmap("xpm/test.xpm",$window, $color));
 	$button->signal_connect('clicked', sub {$toolbar->set_orientation('vertical')});
 
 	$toolbar->append_space();
 
 	$button = $toolbar->append_item( "Icons","Only show toolbar icons",
-		"Toolbar/IconsOnly", new_pixmap("test.xpm",$window, $color));
+		"Toolbar/IconsOnly", new_pixmap("xpm/test.xpm",$window, $color));
 	$button->signal_connect('clicked', sub {$toolbar->set_style('icons')});
 	$button = $toolbar->append_item( "Text","Only show toolbar text",
-		"Toolbar/TextOnly", new_pixmap("test.xpm",$window, $color));
+		"Toolbar/TextOnly", new_pixmap("xpm/test.xpm",$window, $color));
 	$button->signal_connect('clicked', sub {$toolbar->set_style('text')});
 	$button = $toolbar->append_item( "Both","Show toolbar icons and text",
-		"Toolbar/Both", new_pixmap("test.xpm",$window, $color));
+		"Toolbar/Both", new_pixmap("xpm/test.xpm",$window, $color));
 	$button->signal_connect('clicked', sub {$toolbar->set_style('both')});
 
 	$toolbar->append_space;
@@ -897,19 +899,19 @@ sub make_toolbar {
 	$toolbar->append_widget($entry, "This is an unusable GtkEntry ;)", "Hey don't click me!!!");
 
 	$button = $toolbar->append_item( "Small","Use small spaces",
-		"Toolbar/Small",, new_pixmap("test.xpm",$window, $color));
+		"Toolbar/Small",, new_pixmap("xpm/test.xpm",$window, $color));
 	$button->signal_connect('clicked', sub {$toolbar->set_space_size(5)});
 	$button = $toolbar->append_item( "Big","Use big spaces",
-		"Toolbar/Big", new_pixmap("test.xpm",$window, $color));
+		"Toolbar/Big", new_pixmap("xpm/test.xpm",$window, $color));
 	$button->signal_connect('clicked', sub {$toolbar->set_space_size(10)});
 
 	$toolbar->append_space();
 
 	$button = $toolbar->append_item( "Enable","Enable tooltips",
-		undef, new_pixmap("test.xpm",$window, $color));
+		undef, new_pixmap("xpm/test.xpm",$window, $color));
 	$button->signal_connect('clicked', sub {$toolbar->set_tooltips(1)});
 	$button = $toolbar->append_item( "Disable","Disable tooltips",
-		undef, new_pixmap("test.xpm",$window, $color));
+		undef, new_pixmap("xpm/test.xpm",$window, $color));
 	$button->signal_connect('clicked', sub {$toolbar->set_tooltips(0)});
 
 	$toolbar;
@@ -1568,7 +1570,7 @@ sub create_pixmap {
 		
 		$style = $button->get_style;
 
-		($pixmap,$mask) = Gtk::Gdk::Pixmap->create_from_xpm($pixmap_window->window, $style->bg('normal'), "test.xpm");
+		($pixmap,$mask) = Gtk::Gdk::Pixmap->create_from_xpm($pixmap_window->window, $style->bg('normal'), "xpm/test.xpm");
       	$pixmapwid = new Gtk::Pixmap $pixmap, $mask;
       	
       	$label = new Gtk::Label "Pixmap test\n";
@@ -2019,7 +2021,7 @@ sub add1000_clist
 {
   my($widget, $clist)= @_;
   
-  my($pixmap, $mask) = Gtk::Gdk::Pixmap->create_from_xpm($clist->clist_window, $clist->style->white, "test.xpm");
+  my($pixmap, $mask) = Gtk::Gdk::Pixmap->create_from_xpm($clist->clist_window, $clist->style->white, "xpm/test.xpm");
 
   my $row_nr = 0;
 
@@ -2353,6 +2355,28 @@ sub create_file_selection {
 	}
 }
 
+sub font_selection_ok {
+	my($widget, $font) = @_;
+	Gtk->print( $font->get_font_name . "\n");
+}
+
+sub create_font_selection {
+	if (not defined $font_window) {
+		$font_window = new Gtk::FontSelectionDialog "Font Selection Dialog";
+		$font_window->position(-mouse);
+		$font_window->signal_connect("destroy", \&destroy_window, \$font_window);
+		$font_window->signal_connect("delete_event", \&destroy_window, \$font_window);
+		$font_window->ok_button->signal_connect("clicked", \&font_selection_ok, $font_window);
+		$font_window->cancel_button->signal_connect("clicked", sub { destroy $font_window });
+		
+	}
+	if (!visible $font_window) {
+		show $font_window;
+	} else {
+		destroy $font_window;
+	}
+}
+
 sub create_range_controls {
 	my($box1,$box2,$button,$scrollbar,$scale,$separator,$adjustment);
 	
@@ -2921,13 +2945,28 @@ sub create_panes {
 	}
 }
 
-sub progress_timeout {
+
+sub progress_timeout_1_0 {
 	my($progressbar) = @_;
 	my($new_val) = $progressbar->percentage;
+	$new_val += 0.02;
 	if ($new_val>=1.0) {
 		$new_val = 0.0;
 	}
+	
+	$progressbar->update($new_val);
+	
+	return 1;
+}
+
+
+sub progress_timeout_1_1 {
+	my($progressbar) = @_;
+	my($new_val) = $progressbar->get_current_percentage;
 	$new_val += 0.02;
+	if ($new_val>=1.0) {
+		$new_val = 0.0;
+	}
 	
 	$progressbar->update($new_val);
 	
@@ -2967,7 +3006,7 @@ sub create_progress_bar {
 		$vbox->pack_start($pbar,1,1,0);
 		$pbar->show;
 		
-		$progress_timer = Gtk->timeout_add(100, \&progress_timeout, $pbar);
+		$progress_timer = Gtk->timeout_add(100, $gtk_1_0 ? \&progress_timeout_1_0 : \&progress_timeout_1_1, $pbar);
 		
 		$button = new Gtk::Button "close";
 		$button->signal_connect("clicked", sub { destroy $p_window});
@@ -3633,6 +3672,7 @@ sub create_main_window {
 		'dnd',				undef, #\&create_dnd,
 		'entry',			\&create_entry,
 		'file selection',	\&create_file_selection,
+		'font selection',	($gtk_1_0 ? undef : \&create_font_selection),
 		'gamma curve',		\&create_gamma_curve,
 		'handle box',		\&create_handlebox,
 		'list',				\&create_list,
